﻿using Common;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Http;
using TGood.Xian.LogManager;
using ThoughtWorks.QRCode.Codec;
using WxPayAPI.Models;
using wxWebAPI.Models;

namespace WxPayAPI
{
    public class OperateController : ApiController
    {
        /// <summary>
        /// 所有接口都有的必要参数（companyID：厂商ID，userID：用户ID，即手机号，randomStr：随机字符串，sign：签名）
        /// </summary>
        private string[] paramCommon = new string[] { "companyID", "appID", "randomStr", "sign" };


        /// <summary>
        /// 获取设备状态
        /// </summary>
        /// <returns>设备的状态</returns>
        [HttpPost, HttpOptions]
        public JObject StatusQuery([FromBody] SortedDictionary<string, object> request)
        {
            JObject result = new JObject();
            int functionID = 100001;
            
            Sign paramSign = new Sign(request);
            LogEvent.WebAPILoger.Info($"OperateController - StatusQuery, parameter: {paramSign.ToUrl()}");

            //无请求参数，返回POST_DATA_EMPTY
            if (request == null || request.Count == 0)
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.POST_DATA_EMPTY);
                return result;
            }
            try
            {
                string[] paramController = new string[] { "deviceID" };                    //接口其他必要参数

                //参数、签名、权限校验过滤
                result = Filter(paramSign, paramController, functionID);
                if (result.HasValues)
                {
                    return result;
                }

                //从字典中取参数
                string deviceID = paramSign.ToString("deviceID");

                var chargeLogic = new ChargeLogic();
                var status = chargeLogic.GetDeviceStatus(deviceID);
                result.RemoveAll();
                Sign response = new Sign();
                response.SetKey("2ee88adfe868e177db1d63c19778c311");
                string randomStr = GenerateCheckCode();
                response.SetValue("return_code", "SUCCESS");
                response.SetValue("return_msg", null);
                response.SetValue("result_code", "SUCCESS");
                response.SetValue("companyID", paramSign.ToString("companyID"));
                response.SetValue("appID", paramSign.ToString("appID"));
                response.SetValue("deviceID", paramSign.ToString("deviceID"));
                response.SetValue("status", status);
                response.SetValue("randomStr", randomStr);
                response.SetValue("sign", response.MakeSign());
                result = response.ToJObject();
            }
            catch (Exception ex)
            {
                string errorPara = $"OperateController - StatusQuery Error, parameter: {paramSign.ToUrl()}";
                LogEvent.WebAPILoger.Error(errorPara, ex);
                result.RemoveAll();
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SYSTEM_ERROR);
            }
            return result;
        }


        /// <summary>
        /// 获取充电实时数据
        /// </summary>
        /// <returns>充电实时数据</returns>
        [HttpPost, HttpOptions]
        public JObject RealDataQuery([FromBody] SortedDictionary<string, object> request)
        {
            JObject result = new JObject();
            int functionID = 100002;

            Sign paramSign = new Sign(request);
            LogEvent.WebAPILoger.Info($"OperateController - RealDataQuery, parameter: {paramSign.ToUrl()}");

            //无请求参数，返回POST_DATA_EMPTY
            if (request == null || request.Count == 0)
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.POST_DATA_EMPTY);
                return result;
            }
            try
            {
                string[] paramController = new string[] { "deviceID" };                    //接口其他必要参数

                //参数、签名、权限校验过滤
                result = Filter(paramSign, paramController, functionID);
                if (result.HasValues)
                {
                    return result;
                }

                //从字典中取参数
                string deviceID = paramSign.ToString("deviceID");

                var chargeLogic = new ChargeLogic();
                var deviceInfo = chargeLogic.GetRealDataByDeviceCode(deviceID);
                result.RemoveAll();
                Sign response = new Sign();
                response.SetKey("2ee88adfe868e177db1d63c19778c311");
                string randomStr = GenerateCheckCode();
                response.SetValue("return_code", "SUCCESS");
                response.SetValue("return_msg", null);
                response.SetValue("companyID", paramSign.ToString("companyID"));
                response.SetValue("appID", paramSign.ToString("appID"));
                response.SetValue("deviceID", paramSign.ToString("deviceID"));
                if (deviceInfo == null)
                {
                    response.SetValue("result_code", "FAIL");
                    response.SetValue("error_code", ErrorCode.DEVICE_NOT_EXIST);
                    response.SetValue("error_desc", ErrorDesc.DEVICE_NOT_EXIST);
                }
                else
                {
                    response.SetValue("result_code", "SUCCESS");
                    response.SetValue("current", double.Parse(deviceInfo.Current));
                    response.SetValue("voltage", double.Parse(deviceInfo.Voltage));
                    response.SetValue("power", double.Parse(deviceInfo.Power));
                    response.SetValue("SOC", deviceInfo.SOC);
                    response.SetValue("consumption", double.Parse(deviceInfo.PowerConsumption));
                    response.SetValue("startTime", deviceInfo.StartTime);
                    response.SetValue("duration", deviceInfo.Duration);
                    //response.SetValue("remaining", null);
                    response.SetValue("status", deviceInfo.GunStatusName);
                    response.SetValue("orderStatus", Enum.GetName(typeof(EnumWXOrderState), deviceInfo.OrderState));
                    //response.SetValue("toatlFee", deviceInfo.Total_fee);
                    response.SetValue("serviceFee", Math.Round(double.Parse(deviceInfo.ServiceFee) * 100, 0));
                    response.SetValue("electricityFee", Math.Round(double.Parse(deviceInfo.CurrentElectricityFee) * 100, 0));
                    response.SetValue("remaining", deviceInfo.RemainingTime);
                    response.SetValue("updateTime", deviceInfo.LastUpdateTime);
                }
                response.SetValue("randomStr", randomStr);
                response.SetValue("sign", response.MakeSign());
                result = response.ToJObject();
            }
            catch (Exception ex)
            {
                string errorPara = $"OperateController - RealDataQuery Error, parameter: {paramSign.ToUrl()}";
                LogEvent.WebAPILoger.Error(errorPara, ex);
                result.RemoveAll();
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SYSTEM_ERROR);
            }
            return result;
        }


        /// <summary>
        /// 启动充电
        /// </summary>
        [HttpPost, HttpOptions]
        public JObject Start([FromBody] SortedDictionary<string, object> request)
        {
            JObject result = new JObject();
            int functionID = 100003;

            Sign paramSign = new Sign(request);
            LogEvent.WebAPILoger.Info($"OperateController - Start, parameter: {paramSign.ToUrl()}");

            //无请求参数，返回POST_DATA_EMPTY
            if (request == null || request.Count == 0)
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.POST_DATA_EMPTY);
                return result;
            }
            try
            {
                string[] paramController = new string[] { "deviceID", "outTradeID" };                    //接口其他必要参数

                //参数、签名、权限校验过滤
                result = Filter(paramSign, paramController, functionID);
                if (result.HasValues)
                {
                    return result;
                }

                //从字典中取参数
                string companyID = paramSign.ToString("companyID");
                string appID = paramSign.ToString("appID");
                string deviceID = paramSign.ToString("deviceID");
                string outTradeID = paramSign.ToString("outTradeID");

                Regex rgx = new Regex("^[a-zA-Z0-9_\\-\\|#]{0,31}$");
                if (!rgx.IsMatch(outTradeID))
                {
                    result.Add("return_code", "FAIL");
                    result.Add("return_msg", ErrorCode.PARAM_ERROR);
                    return result;
                }

                var chargeLogic = new ChargeLogic();
                var start = chargeLogic.APIStartCharge(companyID, appID, deviceID, outTradeID);
                result.RemoveAll();
                Sign response = new Sign();
                response.SetKey("2ee88adfe868e177db1d63c19778c311");
                string randomStr = GenerateCheckCode();
                response.SetValue("return_code", "SUCCESS");
                response.SetValue("return_msg", null);
                response.SetValue("companyID", paramSign.ToString("companyID"));
                response.SetValue("appID", paramSign.ToString("appID"));
                response.SetValue("deviceID", paramSign.ToString("deviceID"));
                response.SetValue("outTradeID", paramSign.ToString("outTradeID"));
                switch (start)
                {
                    case EnumAPIStart.成功:
                        response.SetValue("result_code", "SUCCESS");
                        response.SetValue("status", "指令已下发");
                        break;
                    case EnumAPIStart.设备未连接:
                        response.SetValue("result_code", "FAIL");
                        response.SetValue("status", "指令未下发");
                        response.SetValue("error_code", ErrorCode.DEVICE_UNUSABLE);
                        response.SetValue("error_desc", ErrorDesc.DEVICE_UNUSABLE);
                        break;
                    case EnumAPIStart.其他原因失败:
                        response.SetValue("result_code", "FAIL");
                        response.SetValue("status", "指令未下发");
                        response.SetValue("error_code", ErrorCode.SYSTEM_ERROR);
                        response.SetValue("error_desc", ErrorDesc.SYSTEM_ERROR);
                        break;
                }
                response.SetValue("randomStr", randomStr);
                response.SetValue("sign", response.MakeSign());
                result = response.ToJObject();
            }
            catch (Exception ex)
            {
                string errorPara = $"OperateController - Start Error, parameter: {paramSign.ToUrl()}";
                LogEvent.WebAPILoger.Error(errorPara, ex);
                result.RemoveAll();
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SYSTEM_ERROR);
            }
            return result;
        }


        /// <summary>
        /// 停止充电
        /// </summary>
        [HttpPost, HttpOptions]
        public JObject Stop([FromBody] SortedDictionary<string, object> request)
        {
            JObject result = new JObject();
            int functionID = 100004;

            Sign paramSign = new Sign(request);
            LogEvent.WebAPILoger.Info($"OperateController - Stop, parameter: {paramSign.ToUrl()}");

            //无请求参数，返回POST_DATA_EMPTY
            if (request == null || request.Count == 0)
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.POST_DATA_EMPTY);
                return result;
            }
            try
            {
                string[] paramController = new string[] { "deviceID" };                    //接口其他必要参数

                //参数、签名、权限校验过滤
                result = Filter(paramSign, paramController, functionID);
                if (result.HasValues)
                {
                    return result;
                }

                //从字典中取参数
                string deviceID = paramSign.ToString("deviceID");

                var chargeLogic = new ChargeLogic();
                var stop = chargeLogic.APIStopCharge(deviceID);
                result.RemoveAll();
                Sign response = new Sign();
                response.SetKey("2ee88adfe868e177db1d63c19778c311");
                string randomStr = GenerateCheckCode();
                response.SetValue("return_code", "SUCCESS");
                response.SetValue("return_msg", null);
                response.SetValue("companyID", paramSign.ToString("companyID"));
                response.SetValue("appID", paramSign.ToString("appID"));
                response.SetValue("deviceID", paramSign.ToString("deviceID"));
                switch (stop)
                {
                    case EnumAPIStop.成功:
                        response.SetValue("result_code", "SUCCESS");
                        response.SetValue("status", "指令已下发");
                        break;
                    case EnumAPIStop.设备不存在:
                        response.SetValue("result_code", "FAIL");
                        response.SetValue("status", "指令未下发");
                        response.SetValue("error_code", ErrorCode.DEVICE_NOT_EXIST);
                        response.SetValue("error_desc", ErrorDesc.DEVICE_NOT_EXIST);
                        break;
                    case EnumAPIStop.设备已离线:
                        response.SetValue("result_code", "FAIL");
                        response.SetValue("status", "指令未下发");
                        response.SetValue("error_code", ErrorCode.DEVICE_OFFLINE);
                        response.SetValue("error_desc", ErrorDesc.DEVICE_OFFLINE);
                        break;
                    case EnumAPIStop.其他原因失败:
                        response.SetValue("result_code", "FAIL");
                        response.SetValue("status", "指令未下发");
                        response.SetValue("error_code", ErrorCode.SYSTEM_ERROR);
                        response.SetValue("error_desc", ErrorDesc.SYSTEM_ERROR);
                        break;
                }
                response.SetValue("randomStr", randomStr);
                response.SetValue("sign", response.MakeSign());
                result = response.ToJObject();
            }
            catch (Exception ex)
            {
                string errorPara = $"OperateController - Stop Error, parameter: {paramSign.ToUrl()}";
                LogEvent.WebAPILoger.Error(errorPara, ex);
                result.RemoveAll();
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SYSTEM_ERROR);
            }
            return result;
        }


        /// <summary>
        /// 获取充电订单
        /// </summary>
        /// <param name="outTradeID">订单号</param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject OrderQuery([FromBody] SortedDictionary<string, object> request)
        {
            JObject result = new JObject();
            int functionID = 100005;

            Sign paramSign = new Sign(request);
            LogEvent.WebAPILoger.Info($"OperateController - OrderQuery, parameter: {paramSign.ToUrl()}");

            //无请求参数，返回POST_DATA_EMPTY
            if (request == null || request.Count == 0)
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.POST_DATA_EMPTY);
                return result;
            }
            try
            {
                string[] paramController = new string[] { "outTradeID" };                    //接口其他必要参数

                //参数、签名、权限校验过滤
                result = Filter(paramSign, paramController, functionID);
                if (result.HasValues)
                {
                    return result;
                }

                //从字典中取参数
                string companyID = paramSign.ToString("companyID");
                string appID = paramSign.ToString("appID");
                string outTradeID = paramSign.ToString("outTradeID");

                var chargeLogic = new ChargeLogic();
                var orders = chargeLogic.GetHistoryOrder(companyID + "_" + appID + "_" + outTradeID).ToList();

                result.RemoveAll();
                Sign response = new Sign();
                response.SetKey("2ee88adfe868e177db1d63c19778c311");
                string randomStr = GenerateCheckCode();
                response.SetValue("return_code", "SUCCESS");
                response.SetValue("return_msg", null);
                response.SetValue("companyID", paramSign.ToString("companyID"));
                response.SetValue("appID", paramSign.ToString("appID"));
                response.SetValue("deviceID", paramSign.ToString("deviceID"));
                response.SetValue("outTradeID", paramSign.ToString("outTradeID"));
                if (orders == null || orders.Count == 0)
                {
                    response.SetValue("result_code", "FAIL");
                    response.SetValue("error_code", ErrorCode.ORDER_NOT_EXIST);
                    response.SetValue("error_desc", ErrorDesc.ORDER_NOT_EXIST);
                }
                else
                {
                    var order = orders.First();
                    response.SetValue("orderID", order.orderid);
                    response.SetValue("station", order.stationname);
                    response.SetValue("startTime", order.starttime);
                    response.SetValue("endTime", order.endtime);
                    response.SetValue("startSOC", order.startsoc);
                    response.SetValue("endSOC", order.endsoc);
                    response.SetValue("consumption", order.power);
                    response.SetValue("piecewise", order.chargepower);
                    response.SetValue("endReason", order.endReasondesc);

                }
                response.SetValue("randomStr", randomStr);
                response.SetValue("sign", response.MakeSign());
                result = response.ToJObject();
            }
            catch (Exception ex)
            {
                string errorPara = $"OperateController - OrderQuery Error, parameter: {paramSign.ToUrl()}";
                LogEvent.WebAPILoger.Error(errorPara, ex);
                result.RemoveAll();
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SYSTEM_ERROR);
            }
            return result;
        }




        //参数过滤
        private JObject Filter(Sign paramSign, string[] paramController, int functionID)
        {
            JObject result = new JObject();
            //参数校验
            result = CheckParam(paramSign, paramController);
            if (result.HasValues)
            {
                return result;
            }
            //签名校验
            result = CheckSign(paramSign);
            if (result.HasValues)
            {
                return result;
            }
            //权限校验
            result = CheckAuthority(paramSign, paramSign.ToString("companyID"), paramSign.ToString("appID"), functionID);
            if (result.HasValues)
            {
                return result;
            }
            return result;
        }

        private JObject Filter(Sign paramSign, string[] paramController)
        {
            JObject result = new JObject();
            //参数校验
            result = CheckParam(paramSign, paramController);
            if (result.HasValues)
            {
                return result;
            }
            //签名校验
            result = CheckSign(paramSign);
            if (result.HasValues)
            {
                return result;
            }
            return result;
        }

        //必要参数校验
        private JObject CheckParam(Sign paramSign, string[] paramController)
        {
            JObject result = new JObject();
            Sign response = new Sign();
            string randomStr = GenerateCheckCode();
            //公共必要参数校验
            for (int i = 0; i < paramCommon.Length; i++)
            {
                if (!paramSign.IsSet(paramCommon[i]))
                {
                    result.Add("return_code", "FAIL");
                    result.Add("return_msg", ErrorCode.LACK_PARAMS);
                    return result;
                }
            }

            //接口必要参数校验
            for (int i = 0; i < paramController.Length; i++)
            {
                if (!paramSign.IsSet(paramController[i]))
                {
                    result.Add("return_code", "FAIL");
                    result.Add("return_msg", ErrorCode.LACK_PARAMS);
                    return result;
                }
            }

            return result;
        }

        //签名校验
        private JObject CheckSign(Sign paramSign)
        {
            JObject result = new JObject();
            Sign response = new Sign();
            string randomStr = GenerateCheckCode();
            ////取私钥，即密码
            //MongoHelper<lco_users> mongo = new MongoHelper<lco_users>();
            //IMongoQuery queryWhere = Query.And(Query.EQ("phone", paramSign.ToString("userID")));
            //lco_users user = mongo.GetSign(queryWhere);
            //if (user == null)
            //{
            //    result.Add("result_code", "FAIL");
            //    result.Add("error_code", ErrorCode.USER_NOAUTH);
            //    result.Add("error_desc", ErrorDesc.USER_NOAUTH);
            //    return result;
            //}

            //设置私钥
            paramSign.SetKey("2ee88adfe868e177db1d63c19778c311");

            //对比签名
            if (paramSign.MakeSign() != paramSign.ToString("sign"))
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.SIGN_ERROR);
                return result;
            }
            return result;
        }

        //权限校验
        private JObject CheckAuthority(Sign paramSign, string companyID, string appID, int functionID)
        {
            JObject result = new JObject();
            Sign response = new Sign();
            string randomStr = GenerateCheckCode();
            //校验厂商权限
            RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
            string appKey = appID + "_" + companyID;
            int[] function = redis.HashGet<int[]>(RedisKey.APP, appKey);
            if (function == null || function.Count() == 0 || !function.Contains(functionID))
            {
                result.Add("return_code", "FAIL");
                result.Add("return_msg", ErrorCode.NOAUTH);
                return result;
            }

            return result;
        }

        //生成随机字符串
        private string GenerateCheckCode()
        {
            string str = string.Empty;
            long num2 = DateTime.Now.Ticks;
            Random random = new Random(((int)(((ulong)num2) & 0xffffffffL)) | ((int)(num2)));
            for (int i = 0; i < 32; i++)
            {
                char ch;
                int num = random.Next();
                if ((num % 2) == 0)
                {
                    ch = (char)(0x30 + ((ushort)(num % 10)));
                }
                else
                {
                    ch = (char)(0x41 + ((ushort)(num % 0x1a)));
                }
                str = str + ch.ToString();
            }
            return str.ToLower();
        }
    }
}
